/*
    QuickMSG
    Copyright (C) 2014  Jeroen Vreeken <jeroen@vreeken.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#define _GNU_SOURCE
#include "contact.h"

#include <string.h>

struct contact {
	int id;
	enum contact_type type;
	enum contact_keystat keystat;
	char *name;
	time_t time_lastact;
	char *address;
	char *members;
	time_t unread;
	int group;
};

struct contact *contact_alloc(void)
{
	return calloc(1, sizeof(struct contact));
}

void contact_free(struct contact *contact)
{
	free(contact->name);
	free(contact->address);
	free(contact->members);
	
	free(contact);
}

int contact_id_set(struct contact *contact, int id)
{
	contact->id = id;
	return 0;
}

int contact_name_set(struct contact *contact, char *name)
{
	if (contact->name)
		free(contact->name);
	
	contact->name = strdup(name);
	
	return 0;
}

int contact_address_set(struct contact *contact, char *address)
{
	if (contact->address)
		free(contact->address);
	
	contact->address = strdup(address);
	if (!contact->name)
		contact->name = strdup(address);
	
	return 0;
}

int contact_members_set(struct contact *contact, char *members)
{
	if (contact->members)
		free(contact->members);
	
	contact->members = strdup(members);
	
	return 0;
}

int contact_members_add(struct contact *contact, char *member)
{
	char *newm;
	
	asprintf(&newm, "%s, %s", contact->members, member);
	free(contact->members);
	contact->members = newm;
	
	return 0;
}

int contact_members_remove(struct contact *contact, char *member)
{
	char *newmembers = malloc(strlen(contact->members)+1);
	char *next = newmembers;
	size_t i, j;
	
	for (i = 0, j = 0; i < contact_members_get_size(contact); i++) {
		char *m = contact_member_get(contact, i);
		
		if (strcmp(member, m)) {
			next += sprintf(next, "%s%s", j > 0 ? ", " : "", m);
			j++;
		}
		
		free(m);
	}
	
	contact_members_set(contact, newmembers);
	
	free(newmembers);
	
	return 0;
}

int contact_type_set(struct contact *contact, enum contact_type type)
{
	contact->type = type;
	return 0;
}

int contact_keystat_set(struct contact *contact, enum contact_keystat keystat)
{
	contact->keystat = keystat;
	return 0;
}

int contact_time_lastact_set(struct contact *contact, time_t t)
{
	contact->time_lastact = t;
	return 0;
}

int contact_unread_set(struct contact *contact, time_t t)
{
	contact->unread = t;
	return 0;
}

int contact_group_set(struct contact *contact, int group)
{
	contact->group = group;
	return 0;
}

int contact_id_get(struct contact *contact)
{
	return contact->id;
}

char *contact_name_get(struct contact *contact)
{
	return contact->name;
}
char *contact_address_get(struct contact *contact)
{
	return contact->address;
}
char *contact_members_get(struct contact *contact)
{
	return contact->members;
}
size_t contact_members_get_size(struct contact *contact)
{
	size_t s = 0;
	size_t i;
	
	if (contact->members) {
		s = 1;
		for (i = 0; i < strlen(contact->members); i++) {
			if (contact->members[i] == ',')
				s++;
		}
	}
	
	return s;
}
char *contact_member_get(struct contact *contact, size_t nr)
{
	char *m = NULL;
	size_t i;
	size_t len;
	char *r;
	
	if (!contact->members)
		return NULL;
	m = contact->members;
	for (i = 0; i < nr; i++) {
		do {
			if (!m[0])
				return NULL;
			m++;
		} while (m[0] != ',');
		i++;
		m++;
	}
	
	while (m[0] == ' ')
		m++;
	
	for (len = 0; m[len] != ',' && m[len]; len++);
	
	r = calloc(1, len+1);
	if (!r)
		return NULL;
	
	memcpy(r, m, len);
	return r;
}

enum contact_type contact_type_get(struct contact *contact)
{
	return contact->type;
}
enum contact_keystat contact_keystat_get(struct contact *contact)
{
	return contact->keystat;
}

time_t contact_time_lastact_get(struct contact *contact)
{
	return contact->time_lastact;
}
time_t contact_unread_get(struct contact *contact)
{
	return contact->unread;
}

int contact_group_get(struct contact *contact)
{
	return contact->group;
}

struct contact_list *contact_list_alloc(void)
{
	return calloc(1, sizeof(struct contact_list));
}

void contact_list_free(struct contact_list *l)
{
	size_t i;
	
	for (i = 0; i < l->size; i++) {
		free(l->contacts[i]);
	}
	
	free(l->contacts);
	free(l);
}


int contact_list_add(struct contact_list *l, struct contact *c)
{
	struct contact **cs;
	size_t s = l->size + 1;
	
	cs = realloc(l->contacts, sizeof(struct contact *) * s);
	if (!cs)
		return -1;
	
	l->contacts = cs;
	l->contacts[l->size] = c;
	l->size = s;
	
	return 0;
}

struct contact *contact_list_remove(struct contact_list *l, size_t c)
{
	struct contact *co = l->contacts[c];
	
	memmove(&l->contacts[c], &l->contacts[c+1], 
	    (l->size - c) * sizeof(struct contact *));
	
	l->size--;
	
	return co;
}

char *contact_print_header(void)
{
	return "id\taddress\tname\ttype\tkeystat\tgroup\tlast_act\tmembers\n";
}

char *contact_print(struct contact *c)
{
	char *p;
	
	asprintf(&p, "%d\t%s\t%s\t%s\t%s\t%d\t%ld\t%s\n",
	    c->id, c->address, c->name,
	    c->type == CONTACT_TYPE_PERSON ? "person" : "group",
	    c->keystat == CONTACT_KEYSTAT_VERIFIED ? "verified" :
	    c->keystat == CONTACT_KEYSTAT_RECEIVED ? "received" : "none",
	    c->group,
	    (long)c->time_lastact,
	    c->members);
	
	return p;
}
