/*
    QuickMSG
    Copyright (C) 2014  Jeroen Vreeken <jeroen@vreeken.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#define _GNU_SOURCE
#include "quickmsg_db.h"
#include "settings.h"
#include "contact.h"
#include "post.h"

#include <stdlib.h>
#include <stdio.h>
#include <sqlite3.h>

#define TABLE_CONTACT		"contacts"
#define KEY_CONTACT_ID		"id"
#define KEY_CONTACT_ADDRESS 	"address"
#define KEY_CONTACT_NAME	"name"
#define KEY_CONTACT_MEMBERS	"members"
#define KEY_CONTACT_TYPE	"type"
#define KEY_CONTACT_KEYSTAT	"keystat"
#define KEY_CONTACT_LASTACT	"lastact"
#define KEY_CONTACT_UNREAD	"unread"
#define KEY_CONTACT_GROUP 	"group_id"

#define CREATE_CONTACT_TABLE \
    "CREATE TABLE IF NOT EXISTS " TABLE_CONTACT " ( " \
    KEY_CONTACT_ID " INTEGER PRIMARY KEY AUTOINCREMENT, " \
    KEY_CONTACT_ADDRESS " TEXT, " \
    KEY_CONTACT_NAME " TEXT, " \
    KEY_CONTACT_MEMBERS " TEXT, " \
    KEY_CONTACT_LASTACT " INTEGER, " \
    KEY_CONTACT_TYPE " INTEGER, " \
    KEY_CONTACT_KEYSTAT " INTEGER, " \
    KEY_CONTACT_UNREAD " INTEGER, " \
    KEY_CONTACT_GROUP " INTEGER )"

#define INSERT_CONTACT \
    "INSERT INTO " TABLE_CONTACT "("\
    KEY_CONTACT_ADDRESS ", " \
    KEY_CONTACT_NAME ", " \
    KEY_CONTACT_MEMBERS ", " \
    KEY_CONTACT_LASTACT ", " \
    KEY_CONTACT_TYPE ", " \
    KEY_CONTACT_KEYSTAT ", " \
    KEY_CONTACT_UNREAD ", " \
    KEY_CONTACT_GROUP ")"

#define UPDATE_CONTACT "UPDATE " TABLE_CONTACT
#define DELETE_CONTACT "DELETE FROM " TABLE_CONTACT

#define TABLE_MESSAGES		"messages"
#define KEY_MESSAGES_ID		"id"
#define KEY_MESSAGES_FROM	"from_id"
#define KEY_MESSAGES_TIME	"time"
#define KEY_MESSAGES_TEXT	"text"
#define KEY_MESSAGES_URI	"uri"

#define CREATE_MESSAGES_TABLE \
    "CREATE TABLE IF NOT EXISTS " TABLE_MESSAGES " ( " \
    KEY_MESSAGES_ID " INTEGER, " \
    KEY_MESSAGES_FROM " INTEGER, " \
    KEY_MESSAGES_TIME " INTEGER, " \
    KEY_MESSAGES_TEXT " TEXT, " \
    KEY_MESSAGES_URI " TEXT  )"

#define INSERT_MESSAGES \
    "INSERT INTO " TABLE_MESSAGES "("\
    KEY_MESSAGES_ID ", " \
    KEY_MESSAGES_FROM ", " \
    KEY_MESSAGES_TIME ", " \
    KEY_MESSAGES_TEXT ", " \
    KEY_MESSAGES_URI ")"

struct quickmsg_db {
	sqlite3 *db;
};


struct quickmsg_db *quickmsg_db_alloc(void)
{
	struct quickmsg_db *db;
	int r;
	char *errmsg = NULL;
	
	db = calloc(1, sizeof(struct quickmsg_db));
	if (!db)
		return NULL;
	
	r = sqlite3_open(settings_db_filename_get(), &db->db);
	if (r) {
		printf("Error opening DB\n");
		goto err;
	}
	
	r = sqlite3_exec(db->db, CREATE_CONTACT_TABLE, NULL, 0, &errmsg);
	if( r != SQLITE_OK ) {
		printf("SQL error: %s\n", errmsg);
		sqlite3_free(errmsg);
	}

	r = sqlite3_exec(db->db, CREATE_MESSAGES_TABLE, NULL, 0, &errmsg);
	if( r != SQLITE_OK ) {
		printf("SQL error: %s\n", errmsg);
		sqlite3_free(errmsg);
	}
	
	return db;

err:
	free(db);
	return NULL;
}

void quickmsg_db_free(struct quickmsg_db *db)
{
	sqlite3_close(db->db);

	free(db);
}

static int callback_contact(void *data, int argc, char **argv, char **col_name)
{
	struct contact_list *l = data;
	
	if (argc >= 9){
		struct contact *c = contact_alloc();
		
		if (argv[0])
			contact_id_set(c, atoi(argv[0]));
		if (argv[1])
			contact_address_set(c, argv[1]);
		if (argv[2])
			contact_name_set(c, argv[2]);
		if (argv[3])
			contact_members_set(c, argv[3]);
		if (argv[4])
			contact_time_lastact_set(c, atol(argv[4]));
		if (argv[5])
			contact_type_set(c, atoi(argv[5]));
		if (argv[6])
			contact_keystat_set(c, atoi(argv[6]));
		if (argv[7])
			contact_unread_set(c, atol(argv[7]));
		if (argv[8])
			contact_group_set(c, atoi(argv[8]));
		
		contact_list_add(l, c);
	}

	return 0;
}

struct contact_list *quickmsg_db_contacts_get(struct quickmsg_db *db, 
    char *where)
{
	struct contact_list *l;
	char *errmsg = NULL;
	char *query;
	int r;
	
	asprintf(&query, "SELECT * FROM " TABLE_CONTACT " %s%s",
		where ? "WHERE " : "",
		where ? where : "");
	
	l = contact_list_alloc();
	
	r = sqlite3_exec(db->db, query, callback_contact, l, &errmsg);
	if( r != SQLITE_OK ) {
		printf("SQL error: %s\n", errmsg);
		sqlite3_free(errmsg);
	}
	
	free(query);
	
	return l;
}

struct contact *quickmsg_db_contact_person_get_by_address(
    struct quickmsg_db *db, char *address)
{
	struct contact_list *l;
	struct contact *c = NULL;
	char *where;
	
	where = sqlite3_mprintf(
	    KEY_CONTACT_TYPE " =  %d AND " 
	    KEY_CONTACT_ADDRESS " = %Q", 
	    CONTACT_TYPE_PERSON, address);
	
	l = quickmsg_db_contacts_get(db, where);
	
	if (l->size)
		c = contact_list_remove(l, 0);
		
	sqlite3_free(where);
	
	return c;
}

struct contact *quickmsg_db_contact_group_get_by_address_and_group(
    struct quickmsg_db *db, char *address, int group)
{
	struct contact_list *l;
	struct contact *c = NULL;
	char *where;
	
	where = sqlite3_mprintf(
	    KEY_CONTACT_TYPE " =  %d AND " 
	    KEY_CONTACT_ADDRESS " = %Q AND "
	    KEY_CONTACT_GROUP " = %d", 
	    CONTACT_TYPE_GROUP, address, group);
	
	l = quickmsg_db_contacts_get(db, where);
	
	if (l->size)
		c = contact_list_remove(l, 0);
		
	sqlite3_free(where);
	
	return c;
}

struct contact *quickmsg_db_contact_get_by_id(
    struct quickmsg_db *db, int id)
{
	struct contact_list *l;
	struct contact *c = NULL;
	char *where;
	
	asprintf(&where, 
	    KEY_CONTACT_ID " =  %d", id);
	
	l = quickmsg_db_contacts_get(db, where);
	
	if (l->size)
		c = contact_list_remove(l, 0);
		
	free(where);
	
	return c;
}

struct contact_list *quickmsg_db_contact_get_all(
    struct quickmsg_db *db)
{
	struct contact_list *l;
	
	l = quickmsg_db_contacts_get(db, NULL);
	
	return l;
}

int quickmsg_db_contact_insert(struct quickmsg_db *db, struct contact *c)
{
	int r = 0;
	char *errmsg = NULL;
	char *query = sqlite3_mprintf(
	    INSERT_CONTACT "  VALUES("
	    "%Q, %Q, %Q, %ld, %d, %d, %ld, %d"
	    ")",
	    contact_address_get(c),
	    contact_name_get(c),
	    contact_members_get(c),
	    contact_time_lastact_get(c),
	    contact_type_get(c),
	    contact_keystat_get(c),
	    contact_unread_get(c),
	    contact_group_get(c));
	
	r = sqlite3_exec(db->db, query, NULL, NULL, &errmsg);
	if( r != SQLITE_OK ) {
		printf("SQL error: %s\n", errmsg);
		sqlite3_free(errmsg);
		r = -1;
	}
	
	sqlite3_free(query);

	return r;
}

int quickmsg_db_contact_update(struct quickmsg_db *db, struct contact *c)
{
	int r = 0;
	char *errmsg = NULL;
	char *query = sqlite3_mprintf(
	    UPDATE_CONTACT "  SET "
	    KEY_CONTACT_ADDRESS "=%Q, "
	    KEY_CONTACT_NAME "=%Q, "
	    KEY_CONTACT_MEMBERS "=%Q, "
	    KEY_CONTACT_LASTACT "=%ld, "
	    KEY_CONTACT_TYPE "=%d, "
	    KEY_CONTACT_KEYSTAT "=%d, "
	    KEY_CONTACT_UNREAD "=%ld, "
	    KEY_CONTACT_GROUP "=%d"
	    " WHERE "
	    KEY_CONTACT_ID "=%d",
	    contact_address_get(c),
	    contact_name_get(c),
	    contact_members_get(c),
	    contact_time_lastact_get(c),
	    contact_type_get(c),
	    contact_keystat_get(c),
	    contact_unread_get(c),
	    contact_group_get(c),
	    contact_id_get(c));
	
	r = sqlite3_exec(db->db, query, NULL, NULL, &errmsg);
	if( r != SQLITE_OK ) {
		printf("SQL error: %s\n", errmsg);
		sqlite3_free(errmsg);
		r = -1;
	}
	
	sqlite3_free(query);

	return r;
}

int quickmsg_db_contact_delete(struct quickmsg_db *db, struct contact *c)
{
	int r = 0;
	char *errmsg = NULL;
	char *query = sqlite3_mprintf(
	    DELETE_CONTACT "  WHERE id = %d",
	    contact_id_get(c));
	
	r = sqlite3_exec(db->db, query, NULL, NULL, &errmsg);
	if( r != SQLITE_OK ) {
		printf("SQL error: %s\n", errmsg);
		sqlite3_free(errmsg);
		r = -1;
	}
	
	sqlite3_free(query);

	return r;
}

int quickmsg_db_post_insert(struct quickmsg_db *db, struct post *p)
{
	int r = 0;
	char *errmsg = NULL;
	char *query = sqlite3_mprintf(
	    INSERT_MESSAGES "  VALUES("
	    "%d, %d, %ld, %Q, %Q"
	    ")",
	    post_id_get(p),
	    post_from_get(p),
	    (long)post_time_get(p),
	    post_text_get(p),
	    post_uri_get(p));
 
	r = sqlite3_exec(db->db, query, NULL, NULL, &errmsg);
	if( r != SQLITE_OK ) {
		printf("SQL error: %s\n", errmsg);
		sqlite3_free(errmsg);
		r = -1;
	}
	
	sqlite3_free(query);

	return r;
}

static int callback_post(void *data, int argc, char **argv, char **col_name)
{
	struct post_list *l = data;
	
	if (argc >= 5){
		struct post *c = post_alloc();
		
		if (argv[0])
			post_id_set(c, atoi(argv[0]));
		if (argv[1])
			post_from_set(c, atoi(argv[1]));
		if (argv[2])
			post_time_set(c, atol(argv[2]));
		if (argv[3])
			post_text_set(c, argv[3]);
		if (argv[4])
			post_uri_set(c, argv[4]);

		post_list_add(l, c);
	}

	return 0;
}

struct post_list *quickmsg_db_posts_get(struct quickmsg_db *db, 
    char *where)
{
	struct post_list *l;
	char *errmsg = NULL;
	char *query;
	int r;
	
	asprintf(&query, "SELECT * FROM " TABLE_MESSAGES " %s%s",
		where ? "WHERE " : "",
		where ? where : "");
	
	l = post_list_alloc();
	
	r = sqlite3_exec(db->db, query, callback_post, l, &errmsg);
	if( r != SQLITE_OK ) {
		printf("SQL error: %s\n", errmsg);
		sqlite3_free(errmsg);
	}
	
	free(query);
	
	return l;
}

struct post_list *quickmsg_db_post_get_by_id(
    struct quickmsg_db *db, int id)
{
	struct post_list *l;
	char *where;
	
	asprintf(&where, 
	    KEY_MESSAGES_ID " =  %d", id);
	
	l = quickmsg_db_posts_get(db, where);
	
	free(where);
	
	return l;
}

